#ifndef TENCENT_H_INCLUDED
#define TENCENT_H_INCLUDED
#include <iostream>
#include <fstream>
#include <cstring>
#include <cstdio>
#include <time.h>
#include "sqlite3.h"



using namespace std;

class ObjBase
{
public:
	static bool CreateTable(const char* databaseFile)
	{
		const char* sqlStr[] =
		{
			"create table user_profile(UserId int, Year_of_birth int, Gender int, Number_of_tweet int);",
			"create table user_profile_detail(UserId int, Tag_Id int);",
			"create table item_category(ItemId int, Item_Category int);",
			"create table item_keyword(ItemId int, Item_Keyword int);",
			"create table user_action(UserId int, Action_Destination_UserId int, Number_of_at_action int,\
				Number_of_retweet int, Number_of_comment int);",
			"create table user_sns(Follower_userid int, Followee_userid int);",
			"create table user_key_word(UserId int, Keyword int);",
			"create table rec_log_train(UserId int, ItemId int, Result int, Unix_timestamp int);",
			"create table rec_log_test(UserId int, ItemId int, Result int, Unix_timestamp int);",
		};

		sqlite3 *db;
		int rc;
		if ((rc = sqlite3_open(databaseFile, &db)) == 0)
		{
			rc += sqlite3_exec(db, "PRAGMA legacy_file_format = TRUE", NULL, NULL, NULL);
			rc += sqlite3_exec(db, "PRAGMA encoding = 'UTF-8'", NULL, NULL, NULL);
			char* pError = NULL;
			for (int i = 0; i < sizeof(sqlStr)/sizeof(char*); i ++)
			{
				if ((rc = sqlite3_exec(db, sqlStr[i], NULL, NULL, &pError)) != 0)
				{
					cout << "sqlite3_exec:" << pError << endl;
					sqlite3_free(pError);
					break;
				}
			}
		}
		else
		{
			cout << "sqlite3_open " << databaseFile << " failed " << rc << endl;
		}
		return rc == 0;
	}
	bool LoadData(const char* dataFile, const char* databaseFile)
	{
		bool result = false;
		ifstream inFile;
		sqlite3 *db;
		time_t totalSeconds = time(NULL);
		inFile.open(dataFile, ifstream::in);
		if (inFile.is_open())
		{
			if (sqlite3_open(databaseFile, &db) == 0)
			{
				int rc = 0;
				rc += sqlite3_exec(db, "PRAGMA synchronous = OFF", NULL, NULL, NULL);
				rc += sqlite3_exec(db, "PRAGMA journal_mode = OFF", NULL, NULL, NULL);
				rc += sqlite3_exec(db, "PRAGMA legacy_file_format = TRUE", NULL, NULL, NULL);
				rc += sqlite3_exec(db, "PRAGMA encoding = 'UTF-8'", NULL, NULL, NULL);
				rc += sqlite3_exec(db, "BEGIN TRANSACTION", NULL, NULL, NULL);

				if (LoadDataHelper(inFile, db))
				{
					result = true;
				}
				rc += sqlite3_exec(db, "END TRANSACTION", NULL, NULL, NULL);
				rc += sqlite3_close(db);
				result = rc == 0;
			}
			inFile.close();
		}
		totalSeconds = time(NULL) - totalSeconds;
		ofstream log("log.txt", ios_base::ate|ios_base::out);
		if (log.is_open())
		{
			log << "LoadData runs in total seconds of " << totalSeconds << endl;
			log.close();
		}
		return result;
	}
protected:
	virtual bool LoadDataHelper(ifstream& inFile, sqlite3* db) = 0;
	ObjBase()
	{

	}
	virtual ~ObjBase()
	{

	}
};

class UserProfile : public ObjBase
{
public:

protected:
	bool LoadDataHelper(ifstream& inFile, sqlite3* db);
};

#endif // TENCENT_H_INCLUDED
